import os,sys,cPickle,math,random
from Bio import SeqIO
import networkx as nx
import matplotlib
matplotlib.use('Agg')
import matplotlib.pylab as P


AA_freq = {'A':8.27, 'Q':3.95, 'L': 9.67, 'S':6.51, 'R':5.53, 'E':6.76, 'K':5.85,'T':5.33, 'N': 4.04, 'G':7.08, 'M':2.42, 'W':1.08, 'D':5.44, 'H':2.27, 'F': 3.86, 'Y':2.91, 'C': 1.36, 'I':5.98, 'P':4.69,'V':6.87}

def ConvertToProbs(freqs):
    probs = {}
    totalCounts = 1.0*sum(freqs.values())
    for key in freqs.keys():
        if totalCounts > 0.0:   
           probs[key] = 1.0*freqs[key]/totalCounts
        else:
           probs[key] = 0.0

    return probs

def GenerateBackgroundDistributionSequence(lSeq):
    aa = AA_freq.keys()
    bgseq = ''
    for idx in range(0,lSeq):
        total = 0
        u1 = random.random()
        for i in aa:
            total += AA_freq[i]/100.00
            if u1 <= total:
               bgseq += i
               break

    if len(bgseq) < lSeq :
        for buf in range(0,lSeq-len(bgseq)):
            u = random.randint(0,19)
            bgseq += AA_freq.keys()[u]
       #print bgseq, lSeq
    return bgseq



class DependenceTree:

     def __init__(self, mi_dict, clFreq=None, pwFreq=None):
         self.MI = mi_dict
         self.MST = None
         self.clfreq = clFreq
         self.pwfreq = pwFreq
         self.clProbs = {}
         self.pwProbs = {}
         self.FullGraph = nx.Graph() 
         self.DiscChouLiu = None
         weight_tuple = [(x[0],x[1],{'weight':-1.0*mi_dict[x]}) for x in mi_dict.keys()]
         self.FullGraph.add_edges_from(weight_tuple) 
 
     def ChouLiuTree(self, whichNodes=[]):
         if len(whichNodes) == 0:
            MST = nx.minimum_spanning_tree(self.FullGraph) 
         else:
            # compute only those edges which are within the nodes....
            new_edge_tuples = [x for x in self.FullGraph.edges(data=True) if (x[0] in whichNodes) and (x[1] in whichNodes)] 
            #print new_edge_tuples
            new_graph = nx.Graph()
            new_graph.add_edges_from(new_edge_tuples)
            MST = nx.minimum_spanning_tree(new_graph)

         print "MST generative: "
         edge_list = MST.edges(data=True)
         #print edge_list
         self.MST = nx.Graph(edge_list)
         #print sorted(list(self.MST))
         #for each edge store the individual nodes and pairwise probabilities
         print "Computing probabilities.."
         #print list(self.MST) 
         for edge in self.MST.edges():
             #edge = edge_tuple[:2]
             #print "Edge number: ", edge
             self.pwProbs.setdefault(edge,ConvertToProbs(self.pwfreq[edge]))
             self.clProbs.setdefault(edge[0],ConvertToProbs(self.clfreq[edge[0]]))
             self.clProbs.setdefault(edge[1],ConvertToProbs(self.clfreq[edge[1]]))
             #pwprob =           

     def ComputeLogLik(self,query_aln,tree='mst',core1Length=0):
         #sum over the individual and pairwise probabilities..
         #assume query alignment is of the same length as the original alignment. 
         nodes = []
         totalLogLik = 0.0
         nullLogLik = 0.0
 
         if tree=='mst':
            MST = self.MST
         else:
            MST = self.DiscChouLiu

         #compute log lik for only some of the edges...
         all_edges_MST = MST.edges(data=True)
         dict_edges = [(x[2]['weight'],(x[0],x[1])) for x in all_edges_MST]
         dict_edges.sort()
         #dict_edges.reverse()
     
         #if (len(query_aln) != (len(all_edges_MST) + 1)):
            #print "Dimensions of query and Tree do not Match!!!"
         #print dict_edges

         max_num_edges = int(1.0*len(dict_edges))

         frac_edges = [x[1] for x in dict_edges[:max_num_edges]] 
         EdgeLogLikelihood = 0.0

         if core1Length >= 0:
            #print "Core 1 Length = ", core1Length
            totalInterfaceEdges = [x for x in frac_edges if ((x[0]<core1Length) and (x[1]>=core1Length))]
            #print totalInterfaceEdges       

         for edge in frac_edges:  
             node0, node1 = edge   
             #only measure the interface edges
             #if not (node0 <= 18 and node1 >= 19) : continue
             aa0 = query_aln[node0].upper()
             aa1 = query_aln[node1].upper()
             #if aa0 == ' ' or aa1 == ' ' : continue
             ##add pairwise
             # the tree distribution is factored into leave marginals times edge probs/node probs
             pwcountMax = sum(self.pwfreq[edge].values())            
             pwNullProb = 1.0/(21.0*pwcountMax)
             scountMax = sum(self.clfreq[node0].values())
             sinNullProb = 1.0/(scountMax)

             #print "TotalCounts: ", pwcountMax, scountMax
  
             currpwprob = self.pwProbs[edge].get((aa0,aa1),pwNullProb)
             if currpwprob <= 0.0:
                currpwprob = pwNullProb

             currPWPROB_NoNorm = currpwprob

             curr1prob = self.clProbs[node0].get(aa0,sinNullProb)
             curr2prob = self.clProbs[node1].get(aa1,sinNullProb)
            
             if curr1prob <= 0.0:
                curr1prob = sinNullProb
             if curr2prob <= 0.0:
                curr2prob = sinNullProb
   
             #if (curr1prob*curr2prob) <= 0.0:
             #print curr1prob, curr2prob, node0, node1, aa0, aa1, currpwprob
   
             currpwprob /= (curr1prob*curr2prob)
             #if tree == 'disc':
                #print curr1prob, curr2prob, node0, node1, aa0, aa1, currpwprob, currPWPROB_NoNorm

             if currpwprob > 0.0:
                totalLogLik += math.log(currpwprob)
                ## Onlye add interface contributions
                if ((node0 < core1Length) and (node1 >= core1Length)):
                   EdgeLogLikelihood += math.log(currpwprob)
             #else:
                ##some high negative number to avoid -inf
                #totalLogLik += -100.00
             
             ## add the singleton probabilities only once
             
             if not (node0 in nodes): 
                currprob = self.clProbs[node0].get(aa0,sinNullProb)

                if currprob == 0.0: 
                   currprob = sinNullProb

   
                if currprob > 0.0:
                   totalLogLik += math.log(currprob)
                   nullLogLik += math.log(currprob)
                #else:
                   #totalLogLik += -100.00
                   #nullLogLik += -100.00
                #print "Singleton: ", node0
                nodes.append(node0)

             if not (node1 in nodes):
                currprob = self.clProbs[node1].get(aa1,sinNullProb)
                if currprob == 0.0: 
                   currprob = sinNullProb

                if currprob > 0.0:
                   totalLogLik += math.log(currprob)
                   nullLogLik += math.log(currprob)
                #else:
                   #totalLogLik += -100.00
                   #nullLogLik += -100.00
                #print "Singleton: ", node1
                nodes.append(node1)
    
         #print "Total Singleton: ", len(nodes)
         #if tree != 'disc':
         print tree, "Total: ", totalLogLik, " InterEdgeContribution: ", EdgeLogLikelihood, " Singleton Contributions: ", nullLogLik, " Num_Interface_Edges: ", len(totalInterfaceEdges)   
         
         return totalLogLik

     def DiscriminativeChouLiuTree(self,nullPwFreq, whichNodes=[]):
             #the modified weights for the edges are as in Sanghavi et al. 2007
             #new weights are given by w{i,j} = \sum{aa_i, aa_j} (P(i,j) - Q(i,j)) log(P(i,j)/P(i)*P(j)) 
             mod_wt_tuple = []
             alphabet = ['I', 'V', 'L', 'F', 'C', 'M', 'A', 'G', 'T', 'S', 'W', 'Y', 'P', 'H', 'E', 'Q', 'D', 'N','K','R','-']
             for edge in self.MI.keys():
                  pwcountMax = sum(self.pwfreq[edge].values())
                  pwNullProb = 1.0/(21.0*pwcountMax)
                  scountMax = sum(self.clfreq[edge[0]].values())
                  sinNullProb = 1.0/(scountMax) 
                  pwProb = ConvertToProbs(self.pwfreq[edge]) 
                  self.pwProbs.setdefault(edge, pwProb)
                  nullpwProb = ConvertToProbs(nullPwFreq[edge])
                  nullpwNullCountMax = sum(nullPwFreq[edge].values())
                  nullpwNullProb = 1.0/(21.0*nullpwNullCountMax)                

                  if not (edge[0] in self.clProbs.keys()):
                     self.clProbs.setdefault(edge[0],ConvertToProbs(self.clfreq[edge[0]]))                      
                  if not (edge[1] in self.clProbs.keys()):
                     self.clProbs.setdefault(edge[1],ConvertToProbs(self.clfreq[edge[1]]))
 
                  w = 0.0
                  for aa1 in alphabet:
                      for aa2 in alphabet:
                          sprob0 = self.clProbs[edge[0]].get(aa1,sinNullProb)
                          if sprob0 == 0.0:
                             sprob0 = sinNullProb
                          sprob1 = self.clProbs[edge[1]].get(aa2,sinNullProb)
                          if sprob1 == 0.0:
                             sprob1 = sinNullProb
                          pwprob_model = pwProb.get((aa1,aa2),pwNullProb)
                          if pwprob_model == 0.0:
                             pwprob_model = pwNullProb
                          nullprob = nullpwProb.get((aa1,aa2),nullpwNullProb)
                          if nullprob == 0.0:
                             nullprob = nullpwNullProb
                          #print pwprob_model, sprob0, sprob1
                          #penalize high infor content
                          #reg_penalty_ij = reg_pen*math.log(pwprob_model/(sprob0*sprob1))
                          
                          #print pwprob_model, nullprob,reg_penalty_ij 
                          w += ((pwprob_model-nullprob)*math.log(pwprob_model/(sprob0*sprob1)))

                  mod_wt_tuple.append((edge[0],edge[1],{'weight':-1.0*w}))
               
             #construct a graph from the modified wt tuples  
             newG = nx.Graph()
             if len(whichNodes) == 0:
                newG.add_edges_from(mod_wt_tuple)
             else:
                print "Doing the Localized version.."
                #use only those nodes in the graph which are within 10 Angs
                new_wt_tuples= [x for x in mod_wt_tuple if (x[0] in whichNodes) and (x[1] in whichNodes)]
                newG.add_edges_from(new_wt_tuples)
             #get mst
             print "MST Discriminative: "
             MST = nx.minimum_spanning_tree(newG)
             edge_list = MST.edges(data=True)
             #print len(edge_list)
             self.DiscChouLiu = nx.Graph(edge_list)


     def ExtractFeaturesFromSequence(self,query_aln,tree='mst'):
         #extract edgewise-features for use in logistic regression
         if tree == 'mst':
            MST = self.MST
         else:
            MST = self.DiscChouLiu    


         all_edges_MST = MST.edges(data=True)
         dict_edges = [(x[2]['weight'],(x[0],x[1])) for x in all_edges_MST]
         dict_edges.sort()
         #dict_edges.reverse()

         #print dict_edges

         max_num_edges = int(1.0*len(dict_edges))

         frac_edges = [x[1] for x in dict_edges[:max_num_edges]]


         ###order of traversal of the edges should be same..
         featureVec = []
         for edge in frac_edges:
             node0, node1 = edge
             aa0 = query_aln[node0].upper()
             aa1 = query_aln[node1].upper()
             #if aa0 == ' ' or aa1 == ' ' : continue
             ##add pairwise
             currpwprob = self.pwProbs[edge][(aa0,aa1)]
             sprob0 = self.clProbs[node0][aa0]
             sprob1 = self.clProbs[node1][aa1]

             if sprob0 == 0.0:
                sprob0 = 1.0/21.0
             if sprob1 == 0.0:
                sprob1 = 1.0/21.0

             if currpwprob == 0.0:
                currpwprob = 1.0/441.0

             currpwprob /= (sprob0*sprob1)
             featureVec.append(currpwprob)
          
         return featureVec    

              
def SplitIntoIndividualStats(miFull, cfFull, pfFull, n_prot1): 
    #split the global stats into two component statistics
    mi1 = {}
    cf1 = {}
    pf1 = {}
    mi2 = {}
    cf2 = {}
    pf2 = {}
    for edge in miFull.keys():
        n1,n2 = edge
        #check if protein 1, i.e if both n1 and n2 are less than the alignment length of core 0
        if n1 < n_prot1 and n2 < n_prot1:
           mi1.setdefault(edge, miFull[edge])
           cf1.setdefault(n1,cfFull[n1])
           cf1.setdefault(n2,cfFull[n2])
           pf1.setdefault(edge,pfFull[edge])
  
        if n1 >= n_prot1 and n2 >= n_prot1:           
           mi2.setdefault(edge, miFull[edge])           
           cf2.setdefault(n1,cfFull[n1])           
           cf2.setdefault(n2,cfFull[n2]) 
           pf2.setdefault(edge,pfFull[edge])
        
    return [(mi1,cf1,pf1),(mi2,cf2,pf2)]
   


if __name__ == '__main__':

 f = open('CoreLengths_All.txt','r')
 flines = f.readlines()
 f.close()

 minIdx = eval(sys.argv[1])

 twoCPXFams = ['c.18.1.1_d.17.5.1_1','c.32.1.1_d.79.2.1_1','b.33.1.1_f.32.1.1_1','c.41.1.1_d.40.1.1_1','c.3.1.3_c.37.1.8_1','d.58.9.1_d.73.1.1_1','a.35.1.5_c.93.1.1_1','c.36.1.10_c.36.1.6_1','a.46.2.1_c.27.1.1_1','a.5.2.1_d.15.1.1_1','g.17.1.2_g.7.1.3_1','d.15.1.1_d.3.1.6_1','b.34.4.4_d.149.1.1_1','c.1.12.1_c.49.1.1_1','b.1.1.1_b.21.1.1_1','d.15.6.1_d.19.1.1_1','d.15.1.1_d.3.1.9_1','b.18.1.4_b.6.1.5_1']


 #random.shuffle(flines)
 
 for reRun in range(minIdx,minIdx+5):
   allModes_NegData_gen = []
   allModes_PosData_gen = []
   allModes_NegData_disc = []
   allModes_PosData_disc = []
    
   
 
   for line in flines:
       mode,core1Length,core2Length = line.strip('\n').split()
       #if mode not in twoCPXFams: continue
       core1Length = eval(core1Length)
       core2Length = eval(core2Length)
       #if (mode != "a.45.1.1_c.47.5.1_1") and (mode != 'c.124.1.2_c.124.1.3_1'): continue
       MIposFile = '/home/rhosur/LocalPPI/SCOPPI_FAMS/MIData/NewMH_MIData/'+mode+'_'+str(reRun)+'_SelectedSamples_train_MI.dat'
       MInegFile = '/home/rhosur/LocalPPI/SCOPPI_FAMS/MIData/NewMH_MIData/'+mode+'_'+str(reRun)+'_SelectedSamples_train_negative_MI.dat'
       
       anyIdx = True 
       #print MIposFile
       #print MInegFile
       if not (os.path.exists(MIposFile) and os.path.exists(MInegFile)): 
          anyIdx = False
          for rer in range(0,9):
              MIposFile = '/home/rhosur/LocalPPI/SCOPPI_FAMS/MIData/NewMH_MIData/'+mode+'_'+str(rer)+'_SelectedSamples_train_MI.dat'
              MInegFile = '/home/rhosur/LocalPPI/SCOPPI_FAMS/MIData/NewMH_MIData/'+mode+'_'+str(rer)+'_SelectedSamples_train_negative_MI.dat'
              if os.path.exists(MIposFile) and os.path.exists(MInegFile):
                 anyIdx = True
                 print "MIFiles found!", MIposFile, MInegFile
                 break

       if not anyIdx : continue
       
       print "Got here"
       records = []
       neg_records = []

       try:
         f = open(MIposFile,'r')
         mi,cf,pf  = cPickle.load(f)
         posSequencesTrain = sum(cf[0].values())
         f.close()
         jointModel= DependenceTree(mi,cf,pf)
         jointModel.ChouLiuTree()
            
         """
         (mi1,cf1,pf1),(mi2,cf2,pf2) = SplitIntoIndividualStats(mi,cf,pf,core1Length)
         core1Model = DependenceTree(mi1,cf1,pf1)
         core1Model.ChouLiuTree()

         core2Model = DependenceTree(mi2,cf2,pf2)
         core2Model.ChouLiuTree()
         """
          
       ##negative tree
         
         f = open(MInegFile,'r')
         mi_neg,cf_neg,pf_neg = cPickle.load(f)
         f.close()
         negSequencesTrain = sum(cf_neg[0].values())
         negModel = DependenceTree(mi_neg,cf_neg,pf_neg)
         negModel.ChouLiuTree()
         
       except:
         print "Could not open MI files!", mode
         continue
       ###build discriminative models for both hypothesis
       
       jointModel.DiscriminativeChouLiuTree(pf_neg) 
       negModel.DiscriminativeChouLiuTree(pf)
     
       #shell1 = range(0,core1Length)
       #shell2 = range(core1Length,core1Length+core2Length)
       #shells = [shell1,shell2]
       #print shells 
       #P.figure()
       #graphFile = mode + '_JTModel_Disc.tree'
       #nx.write_edgelist(jointModel.DiscChouLiu,graphFile)
       #P.savefig(mode+'_JTModel_DiscTree.png')
       
       try:
         ftest = open('./PositiveData/'+mode+'_DBLRAP_MAPK_Aligned.txt','r')
         #ftest = open('./PositiveData/'+mode+'_DBLRAP_SCOPPITWOCPX_Aligned.txt','r')
         records = list(SeqIO.parse(ftest,'stockholm'))
         ftest.close()
   
         
         ftest = open('./NegativeData/'+mode+'_DBLRAP_MAPK_Aligned.txt','r')
         #ftest =  open('./NegativeData/'+mode+'_RandomSCOPPI_Aligned.txt')
         neg_records = list(SeqIO.parse(ftest,'stockholm'))
         ftest.close()
       except:
         print "Could not open test data!", mode
         #neg_records = []
         #continue

       #do not take all the negative data
       random.shuffle(neg_records)
       #negMax = len(neg_records) 
       #flog = open('FeaturesForLogisticRegression_Test20Seqs.dat','w')
          

       neg_loglik = []
       neg_loglik_disc = []
       print "Negative Data Log Liks: " 
       for tr in neg_records:
           #Neg_list = list(tr.seq.tostring())
           #random.shuffle(Neg_list)
           #tr = records[0] 
           if len(tr.seq.tostring()) != core1Length+core2Length:
              print "Dimensions do not match!", mode
              continue

           #core1AlignedPart = list(tr.seq.tostring()[:core1Length])
           #core2AlignedPart = list(tr.seq.tostring()[core1Length:])
           #random.shuffle(core1AlignedPart)
           #random.shuffle(core2AlignedPart)
           #rand_negseq = ''.join(core1AlignedPart+core2AlignedPart)   
           #rand_negseq = ''.join(Neg_list)
           jtLogLik = jointModel.ComputeLogLik(tr.seq.tostring(),'mst',core1Length)
           jtLogLik_disc = jointModel.ComputeLogLik(tr.seq.tostring(),'disc',core1Length)
           
           #core1loglik = core1Model.ComputeLogLik(tr.seq.tostring(),'mst')
           #core2loglik = core2Model.ComputeLogLik(tr.seq.tostring(),'mst') 

           """ 
           ### Skip if the alignments are not good -- to prevent false positives
           core1AlignedPart = tr.seq.tostring()[:core1Length]
           core2AlignedPart = tr.seq.tostring()[core1Length:]
           no_gap_rec1 = ''.join([x for x in core1AlignedPart if x != '-'])
           no_gap_rec2 = ''.join([x for x in core2AlignedPart if x != '-'])
           if len(no_gap_rec1) <= 0.5*len(core1AlignedPart): continue
           if len(no_gap_rec2) <= 0.5*len(core2AlignedPart): continue
           ###################################################################### 
           """

           #jtLogLik = jointModel.ComputeLogLik(tr.seq.tostring())
           #jtLogLik_disc = jointModel.ComputeLogLik(tr.seq.tostring(),'disc')
           #features_neg = jointModel.ExtractFeaturesFromSequence(tr.seq.tostring(),'disc')
           #loglik1 = core1Model.ComputeLogLik(tr.seq.tostring())
           #features_str = [str(x) for x in features_neg]
           #print >> flog,"0",' '.join(features_str)  
           negLogLik = negModel.ComputeLogLik(tr.seq.tostring(),'mst',core1Length)
           negLogLik_disc = negModel.ComputeLogLik(tr.seq.tostring(),'disc',core1Length) 
           #negLogLik = negModel.ComputeLogLik(tr.seq.tostring())
           #negLogLik_disc = negModel.ComputeLogLik(tr.seq.tostring(),'disc')
           #llr = jtLogLik/(jtLogLik+negLogLik)
           #neg_loglik.append(jtLogLik-core1loglik-core2loglik)
           neg_loglik_disc.append(jtLogLik_disc-negLogLik_disc)
           neg_loglik.append(jtLogLik-negLogLik)
           #print "Neg Gen:", mode, tr.id, jtLogLik, negLogLik
           print "Neg Disc:", mode, tr.id, jtLogLik_disc, negLogLik_disc
           
       allModes_NegData_gen += neg_loglik
       allModes_NegData_disc += neg_loglik_disc      

       test_loglik = []
       test_loglik_disc = []
       print "Positive Data Log Liks: "
       for tr in records:
           if len(tr.seq.tostring()) != core1Length+core2Length: 
              print "Dimensions do not match!", mode
              continue

           
           jtLogLik = jointModel.ComputeLogLik(tr.seq.tostring(),'mst',core1Length)
           
           #core1loglik = core1Model.ComputeLogLik(tr.seq.tostring(),'mst')
           #core2loglik = core2Model.ComputeLogLik(tr.seq.tostring(),'mst')
           jtLogLik_disc = jointModel.ComputeLogLik(tr.seq.tostring(),'disc',core1Length)
           #loglik1 = core0Model.ComputeLogLik(tr.seq.tostring())
           #features_pos = jointModel.ExtractFeaturesFromSequence(tr.seq.tostring(),'disc')
           #features_str = [str(x) for x in features_pos]
           #print >> flog,"1", ' '.join(features_str) 
           #loglik2 = core1Model.ComputeLogLik(tr.seq.tostring())
           negLogLik  = negModel.ComputeLogLik(tr.seq.tostring(),'mst',core1Length)
           negLogLik_disc = negModel.ComputeLogLik(tr.seq.tostring(),'disc',core1Length)
           #llr = jtLogLik/(jtLogLik+negLogLik)
           test_loglik.append(jtLogLik-negLogLik)
           test_loglik_disc.append(jtLogLik_disc-negLogLik_disc)
           #print "Pos Gen:", mode, tr.id, jtLogLik, negLogLik
           print "Pos Disc:", mode, tr.id, jtLogLik_disc,negLogLik_disc
 

       allModes_PosData_gen += test_loglik
       allModes_PosData_disc += test_loglik_disc
   #print min(test_loglik), max(test_loglik), min(test_loglik), max(test_loglik_disc)
   #print min(neg_loglik), max(neg_loglik), min(neg_loglik_disc), max(neg_loglik_disc)
   #print test_loglik_disc
 
       bins_pos = [5*x for x in range(0,40)]
       bins_neg = [-5*x for x in range(0,40)]
       bins_neg.reverse()
       bins = bins_neg+bins_pos
   #flog.close()
   #bins.reverse()
       #P.figure()
       #P.hist([neg_loglik,neg_loglik_disc,test_loglik,test_loglik_disc],bins=bins,histtype='bar',label=['-Gen','-Disc','+Gen','+Disc'],color=['blue','green','red','magenta'])
       #P.hist([neg_loglik_disc,test_loglik_disc],bins,label=['Negative Test Data','Positive Test Data'],color=['blue','red'])
       
       #P.legend()
       #P.xlabel('log probability')
       #P.ylabel('# pairs')
       #P.savefig(mode+'_SCOPPIBestTemp_LogLik.png')
   
   
    
   #cutOffRanges = [min(test_loglik_disc)+5*x for x in range(0,50)] 
  
   #print len(allModes_PosData_disc), len(allModes_NegData_disc)
   
   #negMax = 5*len(allModes_PosData_disc)
   #random.shuffle(allModes_NegData_disc)
   #random.shuffle(allModes_NegData_gen)
   #allModes_NegData_disc = allModes_NegData_disc[:negMax]
   #allModes_NegData_gen  = allModes_NegData_gen[:negMax]
 
   spec = []
   spec_disc = []
   sens = []
   sens_disc = []
   for cut in range(0,400,1):
       lkcut = 1.0*cut-200.0
       truePos = [x for x in allModes_PosData_gen if x >= lkcut]
       #truePos_disc = [x for x in allModes_PosData_disc if x >= lkcut] 
       falsePos = [x for x in allModes_NegData_gen if x >= lkcut]
       #falsePos_disc = [x for x in allModes_NegData_disc if x >= lkcut]
       trueNeg = [x for x in allModes_NegData_gen if x < lkcut]
       #trueNeg_disc = [x for x in allModes_NegData_disc if x < lkcut]
       try:
          spec_c = (1.0*len(trueNeg))/len(allModes_NegData_gen)
          spec.append(spec_c)
          sen = (1.0*len(truePos))/len(allModes_PosData_gen)
          sens.append(sen)
       except:  
          pass
       
       
       try: 
          #print lkcut, len(truePos_disc), len(falsePos_disc)
          spec_c_disc =(1.0*len(trueNeg_disc))/len(allModes_NegData_disc)
       #ppv.append(ppv_c) 
          spec_disc.append(spec_c_disc)
       #sen = (1.0*len(truePos))/len(test_loglik)
          sen_disc = (1.0*len(truePos_disc))/len(allModes_PosData_disc)
       #sens.append(sen)
          sens_disc.append(sen_disc) 
       except: 
          pass 
       

   #print ppv_disc
   #print sens_disc
   #P.figure()
   #P.plot(spec,sens,'r')
   #P.ylabel('Sensitivity')
   #P.xlabel('Specificity')
   #P.savefig('MAPKCore_PDBNEG_ROCCurve_'+str(reRun)+'.png')
   
       #print record.id, dt.ComputeLogLik(record.seq.tostring())
   
   """
   for record1 in core0_rec:
       for record2 in core1_rec:
           if not 'sim' in record1.id: continue
           if not 'sim' in record2.id: continue
           ## for true positive assert that they were generated together
           #if (record1.id.split('_')[0] == record2.id.split('_')[0]) and (record1.id.split('_')[2] == record2.id.split('_')[2]):
           #for true negative, they were not generated together, skip if same pdb
           if (record1.id.split('_')[0] == record2.id.split('_')[0]) : continue
           pos_seq1 = record1.seq.tostring()
           pos_seq2 = record2.seq.tostring()
           pos_seq = pos_seq1 + pos_seq2
           #assert len(neg_seq) == len(pos_seq)  
           print record1.id, record2.id, dt.ComputeLogLik(pos_seq) 
   """    
