import os,sys,random
from Bio import SeqIO

class ThreadResults:

      def __init__(self, id1, id2, mutScore, singleScore, gapScore, ssScore, contactScore):
          self.ids = (id1,id2)
          self.mutScore = mutScore
          self.ssScore  = ssScore
          self.gapScore = gapScore
          self.contactScore = contactScore
          self.singleScore = singleScore
          self.isNormalized = False
          if abs(eval(self.mutScore)) < 0.002:
             print self.ids, self.Print()

      def Print(self):
          return ' '.join([self.mutScore, self.ssScore, self.gapScore, self.contactScore, self.singleScore])
   
      def NormalizeScores(self, seqLength):
          #print "SeqLength: ", seqLength
          print "Before Normalizing: ", self.Print()
          if not self.isNormalized:
             self.mutScore = str (1.0*eval(self.mutScore)/seqLength)
             self.ssScore =  str (1.0*eval(self.ssScore)/seqLength)
             self.gapScore = str (1.0*eval(self.gapScore)/seqLength)
             self.contactScore = str (1.0*eval(self.contactScore)/seqLength)
             self.singleScore  = str (1.0*eval(self.singleScore)/seqLength)
             print "After Normalizing: ", self.Print()
             self.isNormalized = True
 

      def getScore(self,sc = None): 
          if not sc:
             print """
                    score type has to be one of:  
                    'mut' --> mutation\
                    'ss' --> sec. str.\
                    'gap' --> gap score \
                    'cont' --> contact score \
                    'sin' --> singleton score "
             """
             return None
             
          else:
                if sc == 'mut': 
                   return eval(self.mutScore)
                elif sc == 'ss': return eval(self.ssScore)
                elif sc == 'gap' : return eval(self.gapScore)
                elif sc == 'cont' : return eval(self.contactScore)
                elif sc == 'sin' : return eval(self.singleScore)
                else:
                     print """"
                            score type has to be one of: 
                            'mut' --> mutation
                            'ss' --> sec. str.
                            'gap' --> gap score
                            'cont' --> contact score
                            'sin' --> singleton score " 
                     """
                     return None
 

f = open(sys.argv[1],'r')
flines = f.readlines()
f.close()

NegativeData =  {}
PositiveData = {}
SizeDict = {}

currFeatures = []
#seqToTemplate = {}

def MapSequenceToTemplate(alnfile):
    f = open(alnfile,'r')
    flines = f.readlines()
    f.close()
    seqToTemplate = {}   
    for line in flines[1:]:
        if 'localPPI' in line: continue
        terms = line.split()
        seqToTemplate.setdefault(terms[2],terms[1])
    return seqToTemplate


def SequenceLength(seqid):
    seqdir = "/home/rhosur/scop1.75_cpxs/SEQ/"
    f = open(seqdir+seqid+'.seq','r')
    record = SeqIO.read(f,"fasta")
    f.close()
    return len(record.seq.tostring())

def MapThreadPairToZscore(threadFile):
    f = open(threadFile,'r')
    flines = f.readlines()
    f.close()
   
    threadPairDict = {}
    # heirarchical dict to store thread results: key1 == seqid, key2 == template id.
    AllthreadResults = {}


    for line in flines:
        if 'NOPAIR' in line:
              # parse to get the thread results
               terms = line.strip('\n').split()
               mscore = terms[15]
               sscore = terms[16]
               gscore = terms[17]
               ssscore = terms[18]
               cscore = terms[20]
               #if '1xbtH18h' in line and 'TK1_P04183' in line:
                  #print mscore, sscore, gscore, ssscore, cscore

               newThreadResults = ThreadResults(terms[0],terms[1],mscore,sscore,gscore,ssscore,cscore)
               #if 'TK1_P04183' in line:
                  #print line, newThreadResults.Print() 

               if AllthreadResults.get(terms[1],None) == None:
                  AllthreadResults.setdefault(terms[1],{})
                  AllthreadResults[terms[1]].setdefault(terms[0],newThreadResults)
               else:
                  if AllthreadResults[terms[1]].get(terms[0],None) == None:
                     AllthreadResults[terms[1]].setdefault(terms[0],newThreadResults)

               #if 'TK1_P04183' in line:
                  #print terms[1], terms[0], AllthreadResults[terms[1]][terms[0]].Print()


        if 'localPPI' in line:
           terms = line.split()
           template, seqpair = terms[0], terms[1]
           z1,z2 = terms[3], terms[4]
           threadPairDict.setdefault((template,seqpair),(z1,z2))
   
    return threadPairDict, AllthreadResults     
                                        

PosseqPairToTemplate = MapSequenceToTemplate('VenkatesanData_BestTemplatesAligned.txt')
PosZscoresDict, PosThreadResultsDict = MapThreadPairToZscore('VenkatesanData_result.dat')
NegseqPairToTemplate = MapSequenceToTemplate('VenkatesanData_BestTemplatesAligned.txt')
NegZscoresDict,NegThreadResultsDict = MapThreadPairToZscore('VenkatesanData_result.dat')

for line in flines:
    """
    skip over the text lines
    """
    if 'MST' in line: continue
    if 'Could' in line : continue
    if 'Computing' in line: continue 
    if 'Neg Disc:' in line:
       terms = line.split()
       mode = terms[2]
       seqid = terms[3]
       #jtloglik = terms[4]
       #core1loglik = terms[5]
       #core2loglik = terms[6] 
       seq1, seq2 = seqid.split('+')
       len1 = SequenceLength(seq1)
       len2 = SequenceLength(seq2)
       try: 
          tmp = NegseqPairToTemplate[seqid]
       except:
          print "Template not found!!", seqid
          continue
       tmp1, tmp2 = tmp.split('-')
       z1, z2 = NegZscoresDict[(tmp,seqid)]
       currResults1 = NegThreadResultsDict[seq1][tmp1]
       #print tmp1, seq1, currResults1.Print()
       currResults1.NormalizeScores(len1)
       currResults2 = NegThreadResultsDict[seq2][tmp2]
       #print tmp2, seq2, currResults2.Print()
       currResults2.NormalizeScores(len2)
       sumMutNeg = currResults1.getScore('mut') + currResults2.getScore('mut')
       sumSSNeg = currResults1.getScore('ss') + currResults2.getScore('ss')
       sumSinNeg = currResults1.getScore('sin')+ currResults2.getScore('sin')
       sumGapNeg = currResults1.getScore('gap')+ currResults2.getScore('gap')
       sumContNeg = currResults1.getScore('cont')+ currResults2.getScore('cont')  
       negFeatureVec = ' '.join([mode, seqid, str(sumMutNeg), str(sumSSNeg), str(sumSinNeg), str(sumGapNeg),str(sumContNeg)]+currFeatures)
       #print "NEG: ", negFeatureVec
       currFeatures = []
       currResults1 = None
       currResults2 = None
       NegativeData.setdefault(seqid,[]).append(negFeatureVec)
    else:
        if 'Pos Disc:' in line:
           terms = line.split()
           mode = terms[2]
           seqid = terms[3]
           seq1, seq2 = seqid.split('+')
           #jtloglik = terms[4]
           #core1loglik = terms[5]
           #core2loglik = terms[6]
           len1 = SequenceLength(seq1)
           len2 = SequenceLength(seq2)
           try:
               tmp = PosseqPairToTemplate[seqid]
           except:
               print "Template not found!!", seqid
               continue

           tmp1, tmp2 = tmp.split('-')
           z1, z2 = PosZscoresDict[(tmp,seqid)]
           currResults1 = PosThreadResultsDict[seq1][tmp1] 
           #print tmp1, seq1, currResults1.Print()
           currResults1.NormalizeScores(len1) 
           
           currResults2 = PosThreadResultsDict[seq2][tmp2]
           #print tmp2, seq2, currResults2.Print()
           currResults2.NormalizeScores(len2)
           length = eval(terms[-1])
           SizeDict.setdefault(mode, length)

           sumMutPos = currResults1.getScore('mut') + currResults2.getScore('mut')
           sumSSPos = currResults1.getScore('ss') + currResults2.getScore('ss')
           sumSinPos = currResults1.getScore('sin') + currResults2.getScore('sin')
           sumContPos = currResults1.getScore('cont') + currResults2.getScore('cont')
           sumGapPos = currResults1.getScore('gap') + currResults2.getScore('gap') 

           posFeatureVec = ' '.join([mode, seqid, str(sumMutPos), str(sumSSPos), str(sumSinPos), str(sumGapPos), str(sumContPos)]+currFeatures)
           #print "POS:", posFeatureVec
           currFeatures = []
           currResults1 = None
           currResults2 = None
           PositiveData.setdefault(seqid,[]).append(posFeatureVec)

        elif 'disc Total:' in line:
          
           scores = line.strip('\n').split()
           totalScore = eval(scores[2])
           edgeScore = eval(scores[4])
           singleScore = eval(scores[7])
           interfaceEdges = eval(scores[-1])
           currFeatures += [str(edgeScore), str(singleScore), str(interfaceEdges)]

#random.shuffle(NegativeData)
#random.shuffle(PositiveData)

print len(NegativeData), len(PositiveData)



for idx in range(0,5):
    fpos = open('NewMHDBLRAPVenkatesanData_LogLikALLData_Positive_AllThreadscores_'+str(idx)+'_disc.txt','w')
    fneg = open('NewMHDBLRAPVenkatesanData_LogLikALLData_Negative_AllThreadscores_'+str(idx)+'_disc.txt','w')

    for seqidPair in NegativeData.keys():
        try:
           data = NegativeData[seqidPair][idx]
           mode = data.split()[0]
           size = SizeDict[mode]
           newData = ' '.join(data.split()[0:-1])
        #u = random.random()
        #if u <= 1.00:
           print >> fneg, newData
        except:
           print "Rerun ", idx, " not found for seqid ", seqidPair
        #else:
        #print >> ftes, '0', data, str(size)

    for seqidPair in PositiveData.keys():

        try:
           data = PositiveData[seqidPair][idx]
           mode = data.split()[0]
           size = SizeDict[mode]
           u = random.random()
           newData = ' '.join(data.split()[0:-1])
        #if u <= 1.00:
           print >> fpos, newData
        except:
           print "Rerun ", idx, " not found for seqid ", seqidPair 
        #else:
        #print >> ftes, '1', data, str(size) 

    fpos.close()
    fneg.close()     
