import os,sys
import random
from Bio import SeqIO

AA = { "ALA" : "A",
       "ASP" : "D",
       "CYS" : "C",
       "GLU" : "E",
       "PHE" : "F",
       "GLY" : "G",
       "HIS" : "H",
       "ILE" : "I",
       "LYS" : "K",
       "LEU" : "L",
       "MET" : "M",
       "ASN" : "N",
       "PRO" : "P",
       "GLN" : "Q",
       "ARG" : "R",
       "SER" : "S",
       "THR" : "T",
       "VAL" : "V",
       "TRP" : "W",
       "TYR" : "Y"}


AA_freq = {'A':8.27, 'Q':3.95, 'L': 9.67, 'S':6.51, 'R':5.53, 'E':6.76, 'K':5.85,'T':5.33, 'N': 4.04, 'G':7.08, 'M':2.42, 'W':1.08, 'D':5.44, 'H':2.27, 'F': 3.86, 'Y':2.91, 'C': 1.36, 'I':5.98, 'P':4.69,'V':6.87}

def GenerateRandomSequence(lSeq):
    #generates a random sequence of length lSeq
    aa = AA.values()
    rndSeq = ''
    for i in range(0,lSeq):
        rndSeq += aa[random.randint(0,19)]

    return rndSeq

def GenerateBackgroundDistributionSequence(lSeq):
    aa = AA_freq.keys()
    bgseq = ''
    for idx in range(0,lSeq):
        total = 0
        u1 = random.random()
        for i in aa:
            total += AA_freq[i]/100.00
            if u1 <= total:
               bgseq += i
               break

    return bgseq


def GenerateBGSequenceSameGapStructure(seq_t):
    aa = AA_freq.keys()
    maxTotal = sum(AA_freq.values())
    newSeq = ''
    for idx in range(0,len(seq_t)):
        if seq_t[idx] == '-':
           newSeq += '-'
        else:
           total = 0
           u1 = random.random()
           for i in aa:
               total += AA_freq[i]/maxTotal
               if u1 <= total:
                  newSeq += i
                  break

    assert len(newSeq) == len(seq_t)

    return newSeq
     
    


def GenerateShuffledSequence(seq):
    seq_list = list(seq)
    random.shuffle(seq_list)
    return ''.join(seq_list)

if __name__=="__main__":
   f = open(sys.argv[1],'r')
   testlines = f.readlines()
   f.close()
   
   for line in testlines: 
       mode = line.split()[0]
       try:
          posFile = open('./PositiveData/'+mode+'_SelectedSamples_test.fasta')
          posRecords = list(SeqIO.parse(posFile,'fasta'))
          posFile.close()
       except:
          print "Coulf not open seq file!", mode
          continue
      
       negFile = open('./NegativeData/'+mode+'_RandomBKG_sameGaps_test.fasta','w')
       for record in posRecords:
           for i in range(0,2):
               negBkgSeq = GenerateBGSequenceSameGapStructure(record.seq.tostring())
               print >> negFile,">",record.id+'_randBKG'+str(i)
               print >> negFile,negBkgSeq

       negFile.close()

   """
   for line in testlines:
       #mode, seq1, seq2 = line.strip('\n').split()
       seq1 = line.strip('\n')
       for seq in [seq1]: 
           outid = seq[:-4] + '_rnd_bg'
           f = open(seq,'r')
           seqs = list(SeqIO.parse(f,'fasta'))
           f.close()
  
           for i in range(1,11):
               f = open(outid+str(i)+'.seq','w')
               print >> f, ">",seqs[0].id 
               print >>f, GenerateBackgroundDistributionSequence(len(seqs[0].seq.tostring()))    
               f.close()

   """ 
   
