#! /usr/bin/env python
import os,sys,math,random
from Bio import AlignIO
from Bio import SeqIO
import cPickle

uniprot_freq = {'A':8.17, 'Q':3.95, 'L':9.67, 'S':6.62,'R':5.50, 'E':6.74, 'K':5.87, 'T':5.34, 'N':4.07, 'G':7.04,'M':2.41,'W':1.09,'D':5.42,
                'H':2.28,'F':3.88,'Y':2.93,'C':1.40,'I':5.94,'P':4.74,'V':6.82}


def GetFrequencies(alignment):
    #number of columns 
    nmax = alignment.get_alignment_length()
    #print nmax
    #number of sequences
    nseq = len(alignment._records)

    global_freq = {'I':0, 'V':0, 'L':0, 'F':0, 'C':0, 'M':0, 'A':0, 'G':0, 'T':0, 'S':0, 'W':0, 'Y':0, 'P':0, 'H':0, 'E': 0, 'Q':0, 'D':0, 'N':0,'K':0,'R':0,'-':0,}

    colWisefreq = []

    for col in range(0,nmax):
        coldict = {'I':0, 'V':0, 'L':0, 'F':0, 'C':0, 'M':0, 'A':0, 'G':0, 'T':0, 'S':0, 'W':0, 'Y':0, 'P':0, 'H':0, 'E': 0, 'Q':0, 'D':0, 'N':0,'K':0,'R':0,'-':0,}
        aln_col = alignment.get_column(col)
        #count occurences
        for res in aln_col:
            if res.upper() in coldict.keys():
               coldict[res.upper()] += 1.0
            else:
               print "Unknown residue ", res

        #skip columns having more than 50% gaps. 
        #if coldict['-'] > 0.5*nseq:
           #colWisefreq.append('NA')
        else:
           colWisefreq.append(coldict)
           #update global counts
           for key in coldict.keys():
               global_freq[key] += coldict[key]



    return global_freq, colWisefreq



def GetPairwiseFrequencies(col_1,col_2):
    
    pw = zip(col_1,col_2)
    count = {} 
    for a1,a2 in pw:
        z = count.setdefault((a1.upper(),a2.upper()),1.0/21.0) 
        count[(a1.upper(),a2.upper())] = z + 1
        #z2 = count.setdefault((a2,a1),1.0/21.0)
        #count[(a2,a1)] = z + 1
       
    return count

def getCounts(file,n_protein1=0):
    f = open(file,'rU')
    msa = AlignIO.read(f,"fasta")
    f.close()
    glFreq, clFreq = GetFrequencies(msa)
    nmax  = msa.get_alignment_length()
    print "NMAX:", nmax
    pw_count = {}
    #sk = 88, #rr = 124. therefore, nmax = 212.
    if n_protein1 == 0:
       n_protein1 = nmax
       n_protein2 = 0
    else:
       n_protein2 = n_protein1

    for sk in range(0,n_protein1):  
        sk_col = msa.get_column(sk)
        sk_freq = clFreq[sk]
        #iterate over the rr columns
        
        for rr in range(n_protein2,nmax):
            ##if within sequence correlations, then keep only one pair
            
            if (sk == rr) and (n_protein2==0): continue                 
            rr_col = msa.get_column(rr)
            rr_freq = clFreq[rr]
            pw_count.setdefault((sk,rr-n_protein2),GetPairwiseFrequencies(sk_col,rr_col))
   
    return pw_count, clFreq


def MutualInformation(pwFreq,clFreq,n_protein1=0, n_pairs=0):
    
    alphabet = ['I', 'V', 'L', 'F', 'C', 'M', 'A', 'G', 'T', 'S', 'W', 'Y', 'P', 'H', 'E', 'Q', 'D', 'N','K','R','-'] 
    pw_alphabet = []
    #print "Number of aligned pairs: ", sum(clFreq[0].values())
    count_max = 21.0*1.0 + sum(clFreq[0].values())
    for i,aa1 in enumerate(alphabet):
        for aa2 in alphabet:
            pw_alphabet.append((aa1,aa2)) 
   
    MI = {}
    #print len(clFreq)
    for sk,rr in pwFreq.keys():
        mutual_info = 0.0
        #iterate over all aa pairs
        for aa1,aa2 in pw_alphabet:
            #normalize the counts to get frequencies
            #print sk,rr,n_protein1
            sk_freq = (clFreq[sk].setdefault(aa1,0.0))/count_max
            rr_freq = (clFreq[rr+n_protein1].setdefault(aa2,0.0))/count_max

            sk_freq += 1.0/count_max
            rr_freq += 1.0/count_max

            #if (sk_freq == 0):
               #sk_freq = 1.0/count_max
            #if rr_freq == 0:
               #rr_freq = 1.0/count_max

            pw_freq = pwFreq[(sk,rr)].setdefault((aa1,aa2),0)
            pw_freq += 1.0/21.0
            
            """
            if pw_freq > 0:
               pw_freq = (pw_freq+1.0/21.0)/count_max
            else:
               pw_freq = pwFreq[(sk,rr)].setdefault((aa2,aa1),1.0/21.0)
            #normalize the counts to get pw freq
            """
            pw_freq = pw_freq/count_max
            
            #if pw_freq <= 0:
               #pw_freq = 1.0/(21.0*count_max)
              
            try: 
               mutual_info += pw_freq*math.log(pw_freq/(sk_freq*rr_freq))
               #if sk == 0 and rr == 0:
               #print sk,rr,aa1, aa2, pw_freq, sk_freq, rr_freq, mutual_info
            except:
               print pw_freq, sk_freq, rr_freq

        MI.setdefault((sk,rr),mutual_info)
     
    return MI           
            


#if __name__ == "__main__":
def ProcessMSA(file,n_samples,n_aln1=None,n_pairs=None):
   #file = "../SK+RR_unique.dat"
   #n_aln1 = 88
   #n_pairs= 2546
   if (not n_aln1) or (not n_pairs):
      n_aln1 = 0
      n_pairs = 0
   pwFreq, clFreq = getCounts(file,n_aln1)
   #print len(pwFreq.keys())
   #print len(pwFreq[(25,30)])
   MI_resampled = MI_Null(file,n_aln1,n_pairs,n_samples)
   MI_prescale = MutualInformation(pwFreq,clFreq,n_aln1,n_pairs)
   MI_scaled = {}
   for key in MI_prescale.keys():
       avgeMI_null = 0.0
       for i in range(n_samples):
           avgeMI_null += MI_resampled[i].get(key,0.0)

       avgeMI_null /= n_samples
       MI_scaled.setdefault(key,MI_prescale[key]-avgeMI_null)

   return MI_scaled, clFreq, pwFreq 
   """
   items = [(v,k) for (k,v) in MutualInfo.items()]
   items.sort()
   items.reverse()
   for i in range(0,60):
       print "SK: ",items[i][1][0], "\t", "RR: ", items[i][1][1], "\t", items[i][0]
   """

def Resample(file,n_aln1=0,n_pairs=0):
    f = open(file,'rU')
    msa = list(SeqIO.parse(f,"fasta"))
    print len(msa)
    f.close()
    f = open("Resampled_data.fasta","w")
    new_list = []
    for i in range(len(msa)):
        idx = random.randint(0,len(msa)-1)
        #print idx
        new_list.append(msa[idx])
    SeqIO.write(new_list,f,"fasta") 
    f.close()


def MI_Null(file,n_aln1,n_pairs,n_random):
    ###shuffle the columns and then calculate MI to get the null distribution.
    f = open(file,'rU')
    msa = AlignIO.read(f,"fasta")
    f.close()
    glFreq, clFreq = GetFrequencies(msa)
    MI_random = []
    nmax  = msa.get_alignment_length()
    if n_aln1 == 0:
       n_aln1 = nmax 
       n_aln2 = 0
    else:
       n_aln2 = n_aln1
    
    for iter in range(0,n_random):
       pw_count= {}
       for sk in range(0,n_aln1):
        sk_col = msa.get_column(sk)
        sk_freq = clFreq[sk]
        #iterate over the rr columns
        for rr in range(n_aln2,nmax):
            rr_col = msa.get_column(rr)
            rr_freq = clFreq[rr]
            sk_col1 = list(sk_col)
            rr_col1 = list(rr_col)
            random.shuffle(sk_col1)
            random.shuffle(rr_col1)
            pw_count.setdefault((sk,rr-n_aln2),GetPairwiseFrequencies(sk_col1,rr_col1))
    
       MI_random.append(MutualInformation(pw_count,clFreq,n_aln2,n_pairs))
   
    return MI_random 

def RelativeEntropies(file,n_aln1,n_pairs):
    f = open(file,'rU')
    msa = AlignIO.read(f,"fasta")
    f.close()
    glFreq, clFreq = GetFrequencies(msa)
    nmax  = msa.get_alignment_length()
    kl_dict = {}
    for idx in range(0,nmax):
        kl = 0.0
        aln_col = clFreq[idx]
        ##for columns having greater than 50% gaps, put KL as zero
        if aln_col['-'] >= 0.50*n_pairs:
           kl_dict.setdefault(idx,0.0)
           continue
        for aa in uniprot_freq.keys():
            p_aa = 1.0*aln_col[aa]/n_pairs
            u_aa = 0.01*uniprot_freq[aa]
            if p_aa > 0.0:
               kl += p_aa*math.log(p_aa/u_aa)
       
        kl_dict.setdefault(idx,kl)

    return kl_dict


if __name__=="__main__":
   #file = "/home/raghavendra/Documents/PPIPrediction/IsoPDZ/PDZInts/src/AlignedSignificantPos_PDZ.dat"
   #n_aln1 = 88
   #n_pairs= 2546
   n_samples = 1
   f = open(sys.argv[1],'r')
   msafile = f.readline().strip('\n')
   f.close()
   file = '/home/bab20/cmapi/Struct_Interactome/HMMPPI/NewMH/' + msafile
   
   outfile = '/groups/berger/MI_DIR/' + msafile[:-6]+'_MI.dat'
   

   MI_scaled, colFreq, pairFreq = ProcessMSA(file,n_samples)
   f = open(outfile,'w')
   cPickle.dump([MI_scaled, colFreq, pairFreq],f,-1)
   f.close()
   """ 
   items = [(v,k) for (k,v) in MI_scaled.items()]
   items.sort()
   items.reverse()
   pairsNR = []
   histMI = {}
   for i in range(0,len(items)):
       if (items[i][1][1],items[i][1][0]) in pairsNR: continue
       #if items[i][0] >= 0.14:
          #print items[i][1], "\t", items[i][0]
       ##3update a histogram -- intervals of 0.05
       idx_hist = int(items[i][0]/0.005)
       num_idx = histMI.setdefault(idx_hist,0)
       histMI[idx_hist] = num_idx + 1
       pairsNR.append(items[i][1])
  
   #cPickle.dump(MI_scaled,f,-1)
   f.close() 
   print histMI
   """
