#!/usr/bin/env python

import os,sys, cPickle
import random,cPickle
from utils import *
from complex import * 
import matplotlib
matplotlib.use('Agg')
import matplotlib.pylab as P
#matplotlib.use('Agg')


VALID_RES =  ['ALA', 'GLY', 'TYR', 'PHE','LEU','ILE','MET','CYS','SER', 'ARG','ASP','GLU','THR','TRP','VAL','GLN','PRO','ASN','LYS','HIS']
fb = open('RBLOSUM64.mat')
blosum62 = cPickle.load(fb)
fb.close()

def ConvertToProbs(freqs):
    probs = {}
    totalCounts = 1.0*sum(freqs.values())
    for key in freqs.keys():
        if totalCounts > 0.0:
           probs[key] = 1.0*freqs[key]/totalCounts
        else:
           probs[key] = 0.0

    return probs

def ContactingPairs(file):
    f =open(file,'r')
    lines = f.readlines()
    f.close()
   
    pairdict = {} 
    for line in lines[5:]:
        terms = line.split()
        res1 = terms[0]
        res2 = terms[3]
        chain1 = terms[2][-1]
        chain2 = terms[5][-1]
        resnum1 = eval(terms[1])
        resnum2 = eval(terms[4])
        if chain2<chain1:
           chain1,chain2 = chain2, chain1
           res1, res2 = res2,res1
           resnum1,resnum2 = resnum2, resnum1
          
        pairdict.setdefault((resnum1,resnum2),(res1,res2))  

    return pairdict

def SimulateInterface(contactDict, sinProb, pweProb,prCon = 0.5,trFeatures=False):
    #default is simulate only half the number of contacts
    simContactDict = {}

    contactsOrder = contactDict.keys()
    contactsOrder.sort()   
 
    #if trFeatures:
    InterfaceFeatureVector = [] 
          

    for contact in contactsOrder: 
        z = random.random() 
        if z <= prCon:
           ##mutate the contact
           ##choose which side should be conditioned on.
           if random.random() <= 0.5:
              ##condition on the first chain ==> sample residue2 from a conditional
              res1,res2 = contactDict[contact]
              idx = 0
              total = 0.0
              MaxVal = 0.0
              #u1 = random.random()
              for j in range(0,20):
                  #P(res2|res1) = P(res1,res2)/P(res1)
                  MaxVal += pweProb[(res1,VALID_RES[j])]/sinProb[res1]
                  ##suppose we sample independently
                  #total += sinProb[VALID_RES[j]]
                  #if u1 < total:
                     #implies choose this residues
                     #print u1, total, contact, res1, res2, VALID_RES[j]
                     #idx = j
                     #break

              u1 = random.uniform(0,MaxVal)
              for j in range(0,20): 
                  total += pweProb[(res1,VALID_RES[j])]/sinProb[res1]
                  if u1<=total:
                     idx = j
                     break

              #print "Core1: ", total
              simContactDict.setdefault(contact,(res1,VALID_RES[idx]))
              intPropensity = pweProb[(res1,VALID_RES[idx])]/(sinProb[res1]*sinProb[VALID_RES[idx]])
              InterfaceFeatureVector.append(intPropensity)
           else:
              ##condition on the second chain ==> sample residue1 from a conditional
              res1,res2 = contactDict[contact]
              idx = 0
              total = 0.0
              MaxVal = 0.0
              #u1 = random.random()
              for j in range(0,20):
                  #P(res1|res2) = P(res1,res2)/P(res2)
                  MaxVal += pweProb[(res2,VALID_RES[j])]/sinProb[res2]
                  #suppose we sample independently
                  #total += sinProb[VALID_RES[j]]
                  #if u1 < total:
                     #implies choose this residues
                     #print u1, total, contact, res2, res1, VALID_RES[j]
                     #idx = j
                     #break
              u1 = random.uniform(0,MaxVal)
              for j in range(0,20): 
                  total += pweProb[(res2,VALID_RES[j])]/sinProb[res2]
                  if u1 <= total:
                     idx = j
                     break

              #print "Core2: ", total
              simContactDict.setdefault(contact,(VALID_RES[idx],res2))
              intPropensity = pweProb[(VALID_RES[idx],res2)]/(sinProb[res2]*sinProb[VALID_RES[idx]])
              InterfaceFeatureVector.append(intPropensity)
                  
           #c1 = VALID_RES[random.randint(0,19)]
           #c2 = VALID_RES[random.randint(0,19)]   
           #simContactDict.setdefault(contact,(c1,c2))
        else:
           simContactDict.setdefault(contact,contactDict[contact])
           res1, res2 = contactDict[contact]
           intPropensity = pweProb[(res1,res2)]/(sinProb[res1]*sinProb[res2])
           InterfaceFeatureVector.append(intPropensity)
    
    if trFeatures: 
       return simContactDict, InterfaceFeatureVector
    else:
       return simContactDict, []       
           

def ParseProbs(file):
    sProb = {}
    pwProb = {}   
   
    f = open(file,'r')
    lines = f.readlines()
    f.close()
    for line in lines:
        if '#' in line :continue
        terms = line.strip('\n').split()
        if len(terms)==2:
           sProb.setdefault(terms[0],eval(terms[1]))
        else:
           if len(terms) > 2:
              res1 = terms[0][2:-2]
              #print res1
              res2 = terms[1][1:-2]
              #print res2
              pwProb.setdefault((res1,res2),eval(terms[2]))
              pwProb.setdefault((res2,res1),eval(terms[2]))
 
    
    #force them to be probabilities
    sProb = ConvertToProbs(sProb)
    pwProb = ConvertToProbs(pwProb)
    print sum(pwProb.values()), sum(sProb.values())
    return sProb, pwProb         

def ChooseAA_Blosum(aa):
    
    bl64_keys = ['A', 'R', 'N', 'D', 'C', 'Q', 'E', 'G', 'H', 'I', 'L', 'K', 'M', 'F', 'P', 'S', 'T', 'W', 'Y', 'V', 'B', 'Z', 'X'] 

    ## given a aa, choose the next aa based on the blosum matrix. This is to simulate non-interface residue evolution
    # From Pulim et al Prot Scie (2008), prel(i,j) = p(i,j)/p(i)*p(j) = 2^(blosum(i,j)/2)
    #therefore, p(i|j) = p(i,j)/p(j) = prel(i,j) * p(i) = 2^(blosum(i,j)/2) * p(i)
 
    idx = 0

    total = 0.0
    MaxVal = 0.0
    aa_s = bl64_keys
    if not aa.upper() in aa_s:
       ##return if gap
       return aa 
 
    #u1 = random.random()
    #to make sure one is transitioning from a probability matrix
    for j in range(0,20): 
        relProb  = 2**(blosum62[(aa.upper(),aa_s[j])]/2)
        sinProb = AA_freq.get(aa_s[j],0.043)/100.00  
        MaxVal += relProb*sinProb
    
    u1 = random.uniform(0,MaxVal)
    #print "MaxVal: ", MaxVal

    for j in range(0,20):       
        relProb  = 2**(blosum62[(aa.upper(),aa_s[j])]/2)
        sinProb = AA_freq.get(aa_s[j],0.043)/100.00
        total += relProb*sinProb    
        if u1 < total:
           idx = j
           break
    
    #print total
    return aa_s[idx]
 
def ConstructSimulatedSequence(alnSeq, simC, sinIntProbs, nonInterfaceMut):
    #alnSeq contains the aligned numbers zipped with the residues
    #simC contains the interface residues with the mutations (simulated)
    # we also need to calculate the transition flux : Q(x[t]|x[t+1])/Q(x[t+1]|x[t])
    # this essentially boils down to product of ratios of the frequencies of non interface amino acids and interface amino acids.
    # also, the previous iteration probs will be in the numerator and the next iteration (t+1) will be in the denominator
    sim_aln = []
    #mutNums = [x[0] for x in simC]   

    transFlux = 0.0

    for aln_num, wt_res in alnSeq:
       if aln_num in simC.keys():
          muRes = AA[simC[aln_num]]
          #print "Mut: ", aln_num, muRes, wt_res
          transFlux += math.log(sinIntProbs[AA_rev[wt_res.upper()]]/sinIntProbs[AA_rev[muRes]])
       else:
          #print aln_num, wt_res
          ##non interface residues should be lower case.
          u2 = random.random()
          if u2 <= nonInterfaceMut: 
             muRes = ChooseAA_Blosum(wt_res).lower()
             #use normal frequencies for non interface resideus
             if wt_res != '-':
                transFlux += math.log(AA_freq[wt_res.upper()]/AA_freq[muRes.upper()])
             #print wt_res, muRes
          else:
             muRes = wt_res.lower()

       sim_aln.append((aln_num,muRes))

    return sim_aln, transFlux 
              
def PrepareFilesForSimEvolOutput(file,outid):
    f = open(file,'r')
    lines = f.readlines()
    f.close()
    
    f = open(file+outid,'w')
    for line in lines:
        ##skip over the end of alignment statement
        if '//' in line : continue
        print >> f, line.strip('\n')

    f.close()
        

def ConstructFullSequence(orig_seq, sim_seq):
    #orig_seq is the dictionary from residuemap function of complex
    # simulated seq is the sequence obtained from ConstructSimulatedSequence
    fullseq = ''
    res_nums = sorted(orig_seq.keys())
    sim_dict = dict(sim_seq)
    for res in res_nums:
        #check to see if this is simulated
        if res in sim_dict.keys():
           wtRes = sim_dict[res]
        else:
           wtRes = orig_seq[res].lower()

        fullseq += wtRes
    
    return fullseq  
    


def Acceptance(eval_prev, eval_curr, logQFlux):
    #actually these are negative evalues, so interpret as energy, lower energy is better 
    #use the metropolis acceptanec criterion
    #use the metropolis-hastings acceptanec criterion
    # a1 = P(x[t+1])*Q(x[t]|x[t+1])
    # a2 = P(x[t]) * Q(x[t+1]|x[t])
    # we need to evaluate a = a1*a2
    # logQFlux = log(Q(x[t]|x[t+1])/Q(x[t+1]|x[t])) 

    accept = 0
    RT =  70.00     
    if eval_curr <= (eval_prev + RT*logQFlux):
       #better state, accept
       return 1
    else:
       #not a better state, but still accept with some probability
       prob_cut = math.exp((-eval_curr+eval_prev)/RT) * math.exp(logQFlux)
       #print "Bad move:", eval_curr, eval_prev, prob_cut
       u = random.random()
       if u <= prob_cut:
          #print "Accepting bad move!"
          return 1
       else:
          return 0 



def ColumnEntropy(col_dict):
    #compute the relative entropy for this column with respect to the background distribution.
  
    kl = 0.0
    n_samples = sum(col_dict.values())
    for aa in AA_freq.keys():
        p_aa = (col_dict.get(aa.upper(),1.0/21.0))/n_samples
        #bg_aa = 0.01*AA_freq[aa]
        if p_aa > 0.0:
           kl += p_aa*math.log(p_aa)

    return kl

def GetBestHMM(seq, family):
    print family
    All_hmms = [x for x in os.listdir('/home/bab20/cmapi/Struct_Interactome/SCOP1.75_HMM') if family in x]
    print All_hmms
    min_pp = 100.0 
    best_hmm = ''
    for hmm in All_hmms:
        pp = AlignSeqToHMM(seq,'/home/bab20/cmapi/Struct_Interactome/SCOP1.75_HMM/'+hmm)
        #print hmm, pp
        if pp <= min_pp:
           min_pp = pp
           best_hmm = '/home/bab20/cmapi/Struct_Interactome/SCOP1.75_HMM/'+hmm 

    return best_hmm


#if __name__=="__main__":
def SimulateEvolution(core0_alnFile, core1_alnFile, simSeq, TrFeatures = False,runIDX = ''):
   #contacts = ContactingPairs('2pmtD81-2pmtC1.dimer')
   #print contacts
   #simcontact = SimulateInterface(contacts,0.05)
   print " "
   sinProb, pweProb = ParseProbs('PairwiseProbs_nr50.txt')
   
    
   ##mutation rate
   mut_rate = 0.05
   nonInterfaceMut = 0.05
  

   ##to monitor the convergence of the chain, we need a column wise entropy counts.
   Core0ColumnWiseFreqs = []
   Core1ColumnWiseFreqs = [] 

  
   #how to select this hmms? Best one for each initial point? or the worst one for each initial point -- so that you get a better mixing.. 
   
   #core0_hmm = './SCOP70_1.75/c.124.1.2_d1ooya2.hmm'
   #core1_hmm = './SCOP70_1.75/c.124.1.3_d2ahua1.hmm'
 
    
   pdb_dir = "/home/bab20/cmapi/Struct_Interactome/cmapi/pdb/"
   #core0 = ParseCoreAlignmentFile('a.45.1.1_c.47.1.5_1_core0.aln')
   core0 = ParseCoreAlignmentFile(core0_alnFile)
   #core1 = ParseCoreAlignmentFile('a.45.1.1_c.47.1.5_1_core1.aln')
   core1 = ParseCoreAlignmentFile(core1_alnFile)
  
   
   PrepareFilesForSimEvolOutput(core0_alnFile,'_sim'+runIDX)
   PrepareFilesForSimEvolOutput(core1_alnFile,'_sim'+runIDX)
    
   f0 = open(core0_alnFile+'_sim'+runIDX,'a')
   f1 = open(core1_alnFile+'_sim'+runIDX,'a')
   if TrFeatures:
      mode_path, mode_file  = os.path.split(core0_alnFile)
      mode = '_'.join(mode_file.split('_')[:3]) 
      ftr = open(mode +'_SimTrFeat.txt','w')

   print >> f0, "\n#Simulated Alignments begin from below."
   print >> f1, "\n#Simulated Alignments begin from below."
   
   mode_path, mode_file  = os.path.split(core0_alnFile)
   mode = '_'.join(mode_file.split('_')[:3])

   complex_map = [(cpxfile.split('_')[0],loadComplex(mode_path+'/'+cpxfile)) for cpxfile in os.listdir(mode_path) if cpxfile.endswith('.cpx')]
   complex_map = dict(complex_map)
    
   f_mi = open(mode+'_CLTraining_'+runIDX+'.txt','w')
   print >> f_mi, '# STOCKHOLM 1.0'  


   EvalueChains = []
   AcceptanceResults = [] 
   total_col_entropy_core0 = {}
   total_col_entropy_core1 = {}  

   for key1 in core0.keys():
       chain1_nums = [x[0] for x in core0[key1]]
       for key2 in core1.keys():
           if key1.split('_')[0] == key2.split('_')[0]:
               
              total_col_entropy_core0.setdefault(key1.split('_')[0],[])
              total_col_entropy_core1.setdefault(key2.split('_')[0],[])

              if TrFeatures:
                 print >> ftr, "#TEMPLATE:",key1.split('_')[0]

              chain2_nums = [x[0] for x in core1[key2]]
              pdbstruct = PDB(key1.split('_')[0],pdb_dir)
              chain1 = pdbstruct.model.__getitem__(key1.split('_')[1])
              chain2 = pdbstruct.model.__getitem__(key2.split('_')[1])
              contacts = GetValidDomainContacts(chain1, chain2, chain1_nums, chain2_nums)
  
              currContacts = contacts
              print " "
              aln0_zip = core0[key1]
              aln1_zip = core1[key2]
        
              core0_orig_full = ConstructFullSequence(complex_map[key1.split('_')[0]].residueMap[0],aln0_zip)
              core1_orig_full = ConstructFullSequence(complex_map[key1.split('_')[0]].residueMap[1],aln1_zip)          
              

              core0_hmm = GetBestHMM(core0_orig_full, mode.split('_')[0])
              core1_hmm = GetBestHMM(core1_orig_full, mode.split('_')[1])

              print "Best HMMs for pdb: ", key1.split('_')[0], core0_hmm, core1_hmm  
 
              orig_core0_10 = ''.join([x[1] for x in aln0_zip if x[1] != '-'])
              orig_core1_10 = ''.join([x[1] for x in aln1_zip if x[1] != '-'])
 
              print "Core0_orig: ", ''.join([x[1] for x in aln0_zip if x[1] != '-'])
              print "Core1_orig: ", ''.join([x[1] for x in aln1_zip if x[1] != '-'])

              core0_prevEvalue = -1.0
              core1_prevEvalue = -1.0
    
              for iter in range(0,100*simSeq+1):
                  simcontact, interface_features = SimulateInterface(currContacts,sinProb,pweProb,mut_rate,TrFeatures)
                  sim0_contacts = dict([(x[0],simcontact[x][0]) for x in simcontact.keys()])
                  sim1_contacts = dict([(x[1],simcontact[x][1]) for x in simcontact.keys()])
   
                  #order_cts = currContacts.keys()
                  #order_cts.sort()
                  #for ic,contact in enumerate(order_cts):
                      #print currContacts[contact], simcontact[contact], interface_features[ic] 
               
                  #aln0_zip = currAln0_zip
                  #aln1_zip = currAln1_zip

                  transFlux1 = 0.0
                  transFlux2 = 0.0

   
                  sim0_aln, transFlux1 = ConstructSimulatedSequence(aln0_zip, sim0_contacts, sinProb, nonInterfaceMut)
                  sim1_aln, transFlux2 = ConstructSimulatedSequence(aln1_zip, sim1_contacts, sinProb, nonInterfaceMut)
   
 
                  ##construct full sequences by filling in rest of the sequence (from the original sequence)
                  core0_fullSeq = ConstructFullSequence(complex_map[key1.split('_')[0]].residueMap[0],sim0_aln)
                  core1_fullSeq = ConstructFullSequence(complex_map[key2.split('_')[0]].residueMap[1],sim1_aln)
           
                  #print "Core0: ", core0_fullSeq
                      #print "Core1: ", core1_fullSeq                   
                  
                  #core0_currEvalue, core0_currScore = MeasureSimilarity(core0_orig_full, core0_fullSeq)
                  #core1_currEvalue, core1_currScore = MeasureSimilarity(core1_orig_full, core1_fullSeq)
                  if iter in range(0,100*simSeq+1,1):
                     #print out the simulated alignments
                     #calculate acceptance criterions
                     #sim_core0_10 = ''.join([x[1] for x in sim0_aln if x[1] != '-'])
                     #sim_core1_10 = ''.join([x[1] for x in sim1_aln if x[1] != '-'])

                     #core0_currEvalue, core0_currScore = MeasureSimilarity(core0_orig_full, core0_fullSeq)
                     #core1_currEvalue, core1_currScore = MeasureSimilarity(core1_orig_full, core1_fullSeq)                      

                     core0_currEvalue = AlignSeqToHMM(core0_fullSeq,core0_hmm)
                     core1_currEvalue = AlignSeqToHMM(core1_fullSeq,core1_hmm)

                     #accept1, accept2 = 1,1
                     accept1 = Acceptance(core0_prevEvalue, core0_currEvalue, transFlux1)
                     accept2 = Acceptance(core1_prevEvalue, core1_currEvalue, transFlux2)
       
                     #acceept the move only if both sequences "belong" to the respective families
                     if not (accept1 and accept2) : 
                        AcceptanceResults.append(0.0)
                        #continue
                     else:
                         #accept the move as part of your chain, compute statistics
                         EvalueChains.append((core0_currEvalue,core1_currEvalue))
                         AcceptanceResults.append(1.0)                      

                         print >>f0, key1+'_sim'+str(iter), " ", ''.join([x[1] for x in sim0_aln])
                         print >> f1, key2+'_sim'+str(iter), " ", ''.join([x[1] for x in sim1_aln])
                         print >>f_mi, key1+'_'+key2+'_sim'+str(iter), ''.join([x[1] for x in sim0_aln])+''.join([x[1] for x in sim1_aln])
                         #print "Core0: ", ''.join([x[1] for x in sim0_aln if x[1] != '-']), core0_currEvalue
                         #print "Core1: ", ''.join([x[1] for x in sim1_aln if x[1] != '-']), core1_currEvalue


                         #update frequency counts for each colum in the alignment, useful to monitor mixing of the chains.
                         #core0 update  
                         for pos,alnRes in enumerate(sim0_aln):
                             #make sure the freq count is not empty
                             if len(Core0ColumnWiseFreqs) <= pos:
                                Core0ColumnWiseFreqs.append({})
                             #if alnRes[1] == '-': continue   
                             ##add a pseudo count of 1/21
                             Core0ColumnWiseFreqs[pos].setdefault(alnRes[1].upper(),1.0/21.0)
                             Core0ColumnWiseFreqs[pos][alnRes[1].upper()] += 1.0

                         #core1 update
                         for pos,alnRes in enumerate(sim1_aln):
                             #make sure the freq count is not empty
                            if len(Core1ColumnWiseFreqs) <= pos:
                               Core1ColumnWiseFreqs.append({})
                            #if alnRes[1] == '-': continue
                            ##add a pseudo count of 1/21
                            Core1ColumnWiseFreqs[pos].setdefault(alnRes[1].upper(),1.0/21.0)
                            Core1ColumnWiseFreqs[pos][alnRes[1].upper()] += 1.0

                        
                         ##compute entropy values for the two cores-- sum over entropies of each column in the alignment. 
                         total_col_entropy_core0[key1.split('_')[0]].append(sum([ColumnEntropy(x) for x in Core0ColumnWiseFreqs]))
                         total_col_entropy_core1[key2.split('_')[0]].append(sum([ColumnEntropy(x) for x in Core1ColumnWiseFreqs]))
                         ##calculation check
                         #if iter == 2:
                            #print [ColumnEntropy(x) for x in Core0ColumnWiseFreqs]
                            #print [ColumnEntropy(x) for x in Core1ColumnWiseFreqs]
 
                         if TrFeatures:
                            str_int_feat = [str(x) for x in interface_features]
                            #if len(str_int_feat) != 55:
                            print len(str_int_feat)   
                            print >> ftr, ' '.join(str_int_feat)

                         ##update the mutated sequence as the current sequence   
                         currContacts = simcontact
                
                         aln0_zip = sim0_aln
                         aln1_zip = sim1_aln
                         core0_prevEvalue = core0_currEvalue
                         core1_prevEvalue = core1_currEvalue
   
   #end alignments
   print >> f0, '//'
   print >> f1, '//'
   print >> f_mi, '//'
   f_mi.close()
   f0.close()
   f1.close()

   #print Core0ColumnWiseFreqs 
   #print Core1ColumnWiseFreqs  

   print "Acceptance Rate: ", sum(AcceptanceResults)/len(AcceptanceResults)
   #print "Evalue Chains: ", EvalueChains
   evalue_core0_plt = [x[0] for x in EvalueChains]
   evalue_core1_plt = [x[1] for x in EvalueChains]
   xiters = range(0,len(evalue_core0_plt))
   #print
   #print total_col_entropy_core0
   #print total_col_entropy_core1  
 
   pdbkeys = total_col_entropy_core0.keys()
   
   ##save the entropy figure
   #fplt = open('c.124.1.2_c.124.1.3_BestHMMs.pkl','w')
   #cPickle.dump([total_col_entropy_core0,total_col_entropy_core1,EvalueChains],fplt,-1)
   #fplt.close()
   core0Symbols = ['ro','r+','r,','r_','r.','r^','rx']
   core1Symbols = ['bo','b+','b,','b_','b.','b^','bx']   
 
   P.figure()
   for plidx in range(0,len(pdbkeys)):
       xvals1 = range(0,len(total_col_entropy_core0[pdbkeys[plidx]]))
       #xvals2 = range(0,len(total_col_entropy_core0[pdbkeys[1]]))
       P.plot(xvals1,total_col_entropy_core0[pdbkeys[plidx]],core0Symbols[plidx],total_col_entropy_core1[pdbkeys[plidx]],core1Symbols[plidx],label=pdbkeys[plidx])
       #P.plot(xvals2,total_col_entropy_core0[pdbkeys[1]],'r+',total_col_entropy_core1[pdbkeys[1]],'b+',label=pdbkeys[1])
       
   P.legend()
   P.savefig(mode+'_AbsEntropyPlot_corr'+runIDX+'.png') 
  
   ##save the E value figure
   P.figure()
   P.plot(xiters,evalue_core0_plt,'r+', xiters,evalue_core1_plt,'b+')
   P.savefig(mode+'_Evalues_Iters_corr'+runIDX+'.png')
   #cPickle.dump(EvalueChains,fplot)
   #fplot.close()   
   
   if TrFeatures:
      ftr.close()


if __name__=="__main__":
   num_of_sim = 300
   f = open(sys.argv[1],'r')
   runidx = sys.argv[1].split('_')[1].split('.')[0]

   
   mode_tr = f.readline().strip('\n')
   #testlines = f.readlines()
   print "running sim evol rep: ", runidx, " for family: ", mode_tr
   f.close()

   
   scop175_dir = '/home/bab20/cmapi/Struct_Interactome/SCOP1.75_CPX/filtered/' 
   for mode in os.listdir(scop175_dir):
       if mode != mode_tr : continue
       #mode = line.split()[0]
       print mode
       mode_c0 = scop175_dir+ mode+'/'+mode + '_core0.aln'
       mode_c1 = scop175_dir+ mode+'/'+mode+'_core1.aln'
       if os.path.exists(mode_c0) and os.path.exists(mode_c1): 
          SimulateEvolution(mode_c0,mode_c1, num_of_sim, False, runidx)
   
