import os
uname = os.uname()
if uname[-1] == 'x86_64':
   #import ddp64 as ddp
   import os
else:
   import ddp
   import os

import  math, cPickle

INT_DISTANCE = 10.0
INT_DISTANCE_2 = INT_DISTANCE * INT_DISTANCE
FAR_DISTANCE = 20.0
CA_DISTANCE = 7.7 * 2 + INT_DISTANCE
CA_DISTANCE_2 = CA_DISTANCE * CA_DISTANCE
#TRUE_INT_DISTANCE = 4.0
TRUE_INT_DISTANCE = 4.5
CORE_INT_DISTANCE = 10.0

class ComplexInfo:
    def __init__(self, pdbid, chains):
        self.pdbid = pdbid
        self.chains = chains

    def __str__(self):
        return self.pdbid+self.chains[0]+self.chains[1]

class ContactMap:

    def _distance2(self, a1, a2):
        x1,y1,z1 = a1.get_coord()
        x2,y2,z2 = a2.get_coord()
        dx,dy,dz = x1-x2,y1-y2,z1-z2
        return dx*dx + dy*dy + dz*dz

    def _residue_distance2(self, r1, r2):
        min_d = 9999999.
        for a1 in r1.get_list():
            for a2 in r2.get_list():
                d = self._distance2(a1,a2)
                if d < min_d:
                    min_d = d
        return min_d

    def __init__(self, pdbid = None, chains = None, residues = None):
        self.info = ComplexInfo(pdbid, chains)
        if not residues:
            self.seq = [None, None]
            self.cmap = None
            return
        cmap = []
        # residues[0], seq[0] ==> row
        # residues[1], seq[1] ==> col
        empty_col = [True for x in range(len(residues[1]))]
        empty_row = [True for x in range(len(residues[0]))]
        for i, r1 in enumerate(residues[0]):
            ca1 = None
            if r1.has_id('CA'): ca1 = r1['CA']
            row = []
            for j, r2 in enumerate(residues[1]):
                if ca1:
                    ca2 = None
                    if r2.has_id('CA'): ca2 = r2['CA']
                    if ca2 and self._distance2(ca1, ca2) > CA_DISTANCE_2:
                        row.append(FAR_DISTANCE)
                        continue
                d = self._residue_distance2(r1,r2)
                if d <= INT_DISTANCE_2:
                    row.append(math.sqrt(d))
#                    if d <= 16.:
#                        print chains[0]+'.'+str(r1.get_id()[1]),
#                        print chains[1]+'.'+str(r2.get_id()[1])
                    try:
                        empty_col[j], empty[j+1] = False, False
                    except:
                        pass
                    try:
                        empty_row[i], empty_row[i+1] = False, False
                    except:
                        pass
                else:
                    row.append(FAR_DISTANCE)
            if not empty_row[i]:                
                cmap.append(row)
        self.seq = [None, None]
        self.seq[0]=[r.get_id()[1] for i,r in enumerate(residues[0]) if not empty_row[i]]
        self.seq[1]=[r.get_id()[1] for i,r in enumerate(residues[1]) if not empty_col[i]]
        cmap_new = []
        for row in cmap:
            cmap_new.append([x for i,x in enumerate(row) if not empty_col[i]])
        self.cmap = cmap_new
        
    def matrix(self):
        return self.cmap

    def dimensions(self):
        if len(self.cmap) == 0:
            return [0, 0]
        return [len(self.cmap), len(self.cmap[0])]

    def size(self):
        r, c = self.dimensions()
        return r * c

    def _insertIntGaps(self, intSeq, seq):
        gapped = []
        i = 0
        for x in seq:
            if x == -1 or x == '-':
                gapped.append(-1)
            else:
                gapped.append(intSeq[i])
                i += 1
        return gapped
    
    def intResiduesUsingSeqAlignment(self, complexes, seq1, seq2):
        intSeqs = [[],[]]

        for i, c in enumerate(complexes):
            cmap = c.contactMap.cmap
            intSeq = []
            for row in cmap:
                intSeq.append(sum([x < TRUE_INT_DISTANCE for x in row]))
	    
            intSeqs[0].append(self._insertIntGaps(intSeq, seq1[i]))
            intSeq = []
            for col in range(len(cmap[0])):
                intSeq.append(sum([x[col] < TRUE_INT_DISTANCE for x in cmap]))
            intSeqs[1].append(self._insertIntGaps(intSeq, seq2[i]))
        return intSeqs
        
        
    def intResidues(self, complexes):
        intSeqs = [[],[]]

        for i, c in enumerate(complexes):
            cmap = c.contactMap.cmap
            intSeq = []
            for row in cmap:
                intSeq.append(sum([x < TRUE_INT_DISTANCE for x in row]))
            intSeqs[0].append(self._insertIntGaps(intSeq, self.seqs[0][i]))
            intSeq = []
            for col in range(len(cmap[0])):
                intSeq.append(sum([x[col] < TRUE_INT_DISTANCE for x in cmap]))
            intSeqs[1].append(self._insertIntGaps(intSeq, self.seqs[1][i]))
        return intSeqs

    def updateCoreNums(self, ints1, ints2, nums, pos, n):
        for i in range(len(ints1)):
            if ints1[i] < CORE_INT_DISTANCE and ints2[i] < CORE_INT_DISTANCE:
                if sum(nums) == 0: n += 1
                nums[pos] = n
                nums[pos+1] = n
                return n
        if nums[pos] != 0: n += 1
        nums[pos+1] = 0
        return n

    def findCores(self):
        coreNums = [[0]*len(self.cmap), [0]*len(self.cmap[0])]
        n = pos = 0
        for row1, row2 in zip(self.cmap[:-1], self.cmap[1:]):
            n = self.updateCoreNums(row1, row2, coreNums[0], pos, n)
            pos += 1
        cols = [[r[i] for r in self.cmap] for i in range(len(self.cmap[0]))]
        n = pos = 0
        for col1, col2 in zip(cols[:-1], cols[1:]):
            n = self.updateCoreNums(col1, col2, coreNums[1], pos, n)
            pos += 1
        return coreNums
        
    def saveImage(self, namebase):
        map_file = namebase+'.tmp'
        png_file = namebase+'.png'
        if len(self.cmap) == 0: return
        f = open(map_file, 'w')
        print >> f, 'map\t'+'\t'.join([str(x) for x in range(len(self.cmap[0]))])
        for i, row in enumerate(self.cmap):
            print >> f, str(i)+'\t'+'\t'.join([str(5*x) for x in row])
        f.close()
        os.system('matrix2png -data '+map_file+' -b -bkgcolor white -maxcolor white -mincolor red -missingcolor grey -r -c -size 8:8 -range 20:100 -e > '+png_file)
        os.system('rm '+map_file)

    def save(self, namebase):
        map_file = namebase+'.map'
        f = open(namebase+'.map', 'w')
        cPickle.dump(self, f, -1)
        f.close()

    def _gapped_row(self, row, gc):
        out = []
        for i, c in enumerate(gc):
            if i > 0:
                out.append(row[i-1])
            if c:
                out += [FAR_DISTANCE] * c
        return out
    
    def _align_matrix(self, C, gc, gr):
        C_aligned = []
        blank_row = [FAR_DISTANCE for x in range(len(C[0])+sum(gc))]
        for j, r in enumerate(gr):
            if j > 0:
                C_aligned.append(self._gapped_row(C[j-1], gc))
            for n in range(r):
                C_aligned.append(blank_row)
        return C_aligned

    def _add_gaps(self, seq, gaps):
        gapped = []
        for i, size in enumerate(gaps):
            if i > 0:
                gapped.append(seq[i-1])
            gapped += [-1] * size
        return gapped

    def _average(self, c, d):
        if len(c) != len(d) or len(c[0]) != len(d[0]):
            raise 'Mismatched matrix dimensions'
        avg = []
        for i in range(len(c)):
            row = []
            for j in range(len(c[0])):
                row.append((c[i][j] + d[i][j])/2.0)
            avg.append(row)
        return avg
        
    def align(self, D):
        if not D:
            return self
        gi, gj, gk, gm, distance = ddp.solve(self.cmap, D.cmap)
        #print "ddp solve working.."
	aligned = ContactMap()
        aligned.seqs = [None, None]
        aligned.infos = []
        aligned.cmap = self._average(self._align_matrix(self.cmap, gi, gj),
                                     self._align_matrix(D.cmap, gk, gm))
        if not hasattr(self, 'seqs'):
            aligned.seqs[0] = [self._add_gaps(self.seq[0], gj)]
            aligned.seqs[1] = [self._add_gaps(self.seq[1], gi)]
            aligned.infos = [self.info]
        else:
            aligned.seqs[0] = [self._add_gaps(x, gj) for x in self.seqs[0]]
            aligned.seqs[1] = [self._add_gaps(x, gi) for x in self.seqs[1]]
            aligned.infos = self.infos
        if not hasattr(D, 'seqs'):
            aligned.seqs[0] += [self._add_gaps(D.seq[0], gm)]
            aligned.seqs[1] += [self._add_gaps(D.seq[1], gk)]
            aligned.infos += [D.info]
        else:
            aligned.seqs[0] += [self._add_gaps(x, gm) for x in D.seqs[0]]
            aligned.seqs[1] += [self._add_gaps(x, gk) for x in D.seqs[1]]
            aligned.infos += D.infos
        
        return aligned, distance

def loadContactMap(map_file):
    f = open(map_file, 'r')
    c = cPickle.load(f)
    f.close()
    return c

def _insertIntGaps(intSeq, seq):
    gapped = []
    i = 0
    for x in seq:
        if x == -1 or x == '-':
            gapped.append(-1)
        else:
            gapped.append(intSeq[i])
            i += 1
    return gapped

def intResiduesUsingSeqAlignment(complexes, seq1, seq2):
    intSeqs = [[],[]]

    for i, c in enumerate(complexes):
        cmap = c.contactMap.cmap
        intSeq = []
        for row in cmap:
            intSeq.append(sum([x < TRUE_INT_DISTANCE for x in row]))
            #print x
	intSeqs[0].append(_insertIntGaps(intSeq, seq1[i]))
        #print intSeq
	#print seq1[i]
	intSeq = []
        for col in range(len(cmap[0])):
            intSeq.append(sum([x[col] < TRUE_INT_DISTANCE for x in cmap]))
        #print intSeq
	#print seq2[i]
	intSeqs[1].append(_insertIntGaps(intSeq, seq2[i]))
    return intSeqs

