from Bio.PDB import *
from Bio.PDB.PDBParser import PDBParser
from urllib2 import urlopen
import os, re

PDB_DIR = '/afs/csail.mit.edu/u/r/rhosur/StructAlign/cmapi/pdb'
AA = { "ALA" : "A",
       "ASP" : "D",
       "CYS" : "C",
       "GLU" : "E",
       "PHE" : "F",
       "GLY" : "G",
       "HIS" : "H",
       "ILE" : "I",
       "LYS" : "K",
       "LEU" : "L",
       "MET" : "M",
       "ASN" : "N",
       "PRO" : "P",
       "GLN" : "Q",
       "ARG" : "R",
       "SER" : "S",
       "THR" : "T",
       "VAL" : "V",
       "TRP" : "W",
       "TYR" : "Y"}       

def getNum(res):
    n = ''
    for c in res:
        if c.isdigit():
            n += c
        else:
            break
    return n
        

class PDB:
    def __init__(self, id, pdbdir = PDB_DIR):
        pdb_file = pdbdir+'/'+id+'.pdb'
        if not os.path.exists(pdb_file):
            print 'downloading',id,'...'
            try:
               f = urlopen('http://scop.mrc-lmb.cam.ac.uk/scop/pdb.cgi?pdb='+id+'&disp=pdb')
            except:
               print "HTTP error from SCOP, trying RCSB"
               try:
                  f = urlopen('http://www.rcsb.org/pdb/files/'+id+'.pdb')
               except:
                  print "Error downloading pdb file"
                  return

            data = f.read()
            f.close()
            f = open(pdb_file, 'w')
            if '<head>' in data[:10]:
               f.write(data[47:-6])
            else:
               f.write(data)
            f.close()
            print 'done'
        p = PDBParser(PERMISSIVE=1)
        self.id = id
        self.model = p.get_structure(id, pdb_file)[0]

    def getResidue(self, chain_id, residue_id):
        return self.model[chain_id][int(residue_id)]

    def getFragmentResidues(self, fragment):
        pdb = self.model
        chain, limits = fragment.strip().split(':')
        residues = []
        segs = limits.split(',')
        r = []
        for seg in segs:
            if not seg or seg == '-':
                return [x for x in pdb[chain].get_list() if x.get_id()[0] == ' ']
            else:
                start, end = re.search('(-?\d+)[A-Z]?-(-?\d+)[A-Z]?$', seg).groups()
                r += range(int(start), int(end)+1)
        return [x for x in pdb[chain].get_list() if x.get_id()[1] in r and x.get_id()[0] == ' ']
        
